/*
** Comments v1.1
**
** A WWW comments form
**
** Copyright (c) 9.5.94 by Andreas Ley <ley@rz.uni-karlsruhe.de>
**
** Permission to use, copy, modify, and distribute this software for any
** purpose and without fee is hereby granted, provided that the above
** copyright notice appears in all copies. This software is provided "as is"
** and without any express or implied warranties.
**
** This program has been tested on a HP9000/720 with HP-UX A.08.07
** In this environment, neither lint -u nor gcc -Wall produce any messages.
** If you encounter any errors or need to make any changes to port it
** to another platform, please contact me.
**
** Version history
**
** Version 1.1 - 16.5.94
**	Added mailing capabilities
**
** Version 1.0 - 9.5.94
**	Initial version
*/

char header[]="comments v1.1\n(c) 1994 by Andreas Ley\n";

/* General definitions */

#define	USEMAIL	/* Undef if writing to filesystem */

#define	MASTER	"webmaster@rz.uni-karlsruhe.de"
#ifdef USEMAIL
#define	FROM	"www@rz.uni-karlsruhe.de (" TITLE ")"
#define	TO	"nick@rz.uni-karlsruhe.de"
#define	MAILCMD	"/usr/lib/sendmail -bs >/dev/null"
#else /* USEMAIL */
#define	TZ	"MEZ-1MESZ"
#endif /* USEMAIL */

/* File locations */

#ifndef USEMAIL
#define	ROOT	"/usr/local/etc/httpd/lib/comments"
#define	COUNTER	".counter"
#endif /* USEMAIL */

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/types.h>
#ifndef USEMAIL
#include <time.h>
#endif /* USEMAIL */
#include "comments_german.h"

#define	FALSE	0		/* This is the naked Truth */
#define	TRUE	1		/* and this is the Light */

/* Functions from util.c */

char *makeword(char *line, char stop);
char *fmakeword(FILE *f, char stop, int *len);
void unescape_url(char *url);
void plustospace(char *str);

/* Global variables */

#ifndef USEMAIL
char	root[]=ROOT;
#endif /* USEMAIL */
char	master[]=MASTER;


/* Prints header of a HTML-Request */
void show_header(char *subtitle, char *description)
{
	(void)printf("Content-type: text/html\n\n");
	(void)printf("<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML//EN\">\n");
	(void)printf("<HTML>\n");
	(void)printf("<HEAD>\n");
	(void)printf("<TITLE>%s%s%s</TITLE>\n",TITLE,subtitle?" - ":"",subtitle?subtitle:"");
	(void)printf("<LINK REV=\"made\" HREF=\"mailto:%s\">\n",master);
	(void)printf("</HEAD>\n");
	(void)printf("<BODY>\n");
	(void)printf("<H1>%s</H1>\n",subtitle?subtitle:TITLE);
	if (description) {
		(void)printf(description,master);
		(void)printf("<P>\n");
	}
}



/* Prints trailer of a HTML-Request */
void show_trailer()
{
	(void)printf("</BODY>\n");
	(void)printf("</HTML>\n");
}



/* Shows an error page, based on wrong user input */
void show_error(char *error)
{
	show_header(ERROR,error);
	show_trailer(FALSE);
	exit(0);
}



/* Shows an error page, based on internal misfunction */
void show_fatal(char *error)
{
	show_header(ERROR,error);
	(void)printf(ERROR_FATAL,master);
	(void)printf("<P>\n");
	show_trailer(FALSE);
	exit(0);
}



int tagindex(char **htag, int max, char *tagname)
{
	int	cnt;

	for (cnt=0;cnt<max&&strcmp(htag[cnt],tagname);cnt++);
	return(cnt);
}



char *secure(char *text)
{
	char	*src,*dest;

	if (text) {
		for (src=text,dest=text;*src;src++)
			if ((*src&0x7f)>=0x20)
				*dest++=*src;
		*dest='\0';
	}
	return(text);
}



void usage(char *image)
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v]\n",image);
	exit(1);
}


int main(int argc, char *argv[])
{
	int		c;
	char		buffer[1024],*body,*ptr,*nptr;
	char		*tag,*val,*htag[1024],*hval[1024];
	FILE		*dest;
	int		cl,max,cnt;
#ifndef USEMAIL
	FILE		*src;
	int		id;
	time_t		tloc;
#endif /* USEMAIL */

	while ((c=getopt(argc,argv,"vh?")) != EOF)
		switch ((char)c) {
		case 'v':
			(void)fprintf(stderr,header);
			exit(0);
		case 'h':
			(void)fprintf(stderr,header);
		case '?':
			usage(argv[0]);
		}

	/* If method is GET, draw a form, if POST then parse the form */
	if (strcmp(getenv("REQUEST_METHOD"),"POST")) {
		show_header(NULL,NULL);
		printf(COMMENT_FORM,getenv("SCRIPT_NAME"));
		show_trailer();
	}
	else {

		/* Just to check... */
		if (strcmp(getenv("CONTENT_TYPE"),"application/x-www-form-urlencoded"))
			show_fatal(ERROR_CONTENT_TYPE);

		/* Parse form */
		cl=atoi(getenv("CONTENT_LENGTH"));
		if (cl==0)
			show_fatal(ERROR_CONTENT_LENGTH);
		max=0;
		body=NULL;
		while (cl&&(!feof(stdin))) {
			val=fmakeword(stdin,'&',&cl);
			plustospace(val);
			unescape_url(val);
			tag=makeword(val,'=');
			if (!strcmp(tag,"Body"))
				body=strdup(val);
			else {
				htag[max]=strdup(secure(tag));
				hval[max]=strdup(secure(val));
				max++;
			}
		}

		/* Requirements */
		if (!body||strlen(body)<5)
			show_error(ERROR_NOBODY);
		if ((!*hval[tagindex(htag,max,"Name")])&&(!*hval[tagindex(htag,max,"Reply-To")]))
			show_error(ERROR_ADDRESS);

#ifdef USEMAIL
	/*
	sprintf(buffer,MAILCMD,TO);
	*/
	sprintf(buffer,MAILCMD);
	if ((dest=popen(buffer,"w"))) {
		fprintf(dest,"HELO\n");
		fprintf(dest,"MAIL FROM:<%s>\n",FROM);
		fprintf(dest,"RCPT TO:<%s>\n",TO);
		fprintf(dest,"DATA\n");
		fprintf(dest,"From: %s\n",FROM);
		fprintf(dest,"To: %s\n",TO);
		fprintf(dest,"Subject: %s\n",SUBJECT);
		fprintf(dest,"Reply-To: %s\n",*hval[tagindex(htag,max,"Reply-To")]?hval[tagindex(htag,max,"Reply-To")]:"not-for-mail");
		if ((ptr=getenv("REMOTE_HOST")))
			fprintf(dest,"Host: %s\n",ptr);
		if ((ptr=getenv("REMOTE_USER")))
			fprintf(dest,"User: %s\n",ptr);
		fprintf(dest,"\n");
		for (cnt=0;cnt<max;cnt++)
			if (strcmp(htag[cnt],"Reply-To")&&*hval[cnt])
				fprintf(dest,"%s: %s\n",htag[cnt],hval[cnt]);
		fprintf(dest,"\n");
		ptr=body;
		while ((nptr=strchr(ptr,'\n'))) {
			*nptr++='\0';
			if (*ptr=='.')
				fprintf(dest,".");
			fprintf(dest,"%s\n",secure(ptr));
			ptr=nptr;
		}
		fprintf(dest,"%s\n",secure(ptr));
		fprintf(dest,".\n");
		fprintf(dest,"QUIT\n");
		fclose(dest);
	}
	else
		show_fatal(ERROR_POPEN);
#else /* USEMAIL */
		/* Get next ID */
		(void)putenv("TZ=" TZ);
		if(chdir(root))
			show_fatal(ERROR_DIRECTORY);
		id=1;
		if (src=fopen(COUNTER,"r")) {
			fgets(buffer,sizeof(buffer),src);
			id=atoi(buffer)+1;
			fclose(src);
		}
		if (dest=fopen(COUNTER,"w")) {
			fprintf(dest,"%d\n",id);
			fclose(dest);
		}
		else
			show_fatal(ERROR_WRITE);

		/* Write comment */
		umask(022);
		sprintf(buffer,"%d",id);
		if (dest=fopen(buffer,"w")) {
			time(&tloc);
			for (cnt=0;cnt<max;cnt++)
				if (*hval[cnt])
					fprintf(dest,"%s: %s\n",htag[cnt],hval[cnt]);
			strftime(buffer,sizeof(buffer),"%d %b %Y %H:%M:%S %Z",localtime(&tloc));
			fprintf(dest,"Date: %s\n",buffer);
			if (ptr=getenv("REMOTE_HOST"))
				fprintf(dest,"Host: %s\n",ptr);
			if (ptr=getenv("REMOTE_USER"))
				fprintf(dest,"User: %s\n",ptr);
			fprintf(dest,"\n%s\n",body);
			fclose(dest);
		}
		else
			show_fatal(ERROR_WRITE);
#endif /* USEMAIL */

		/* Drop the user a message */
		show_header(SUCCESS_HEADER,NULL);
		printf(SUCCESS_DESC);
		show_trailer();
	}

	return(0);
}
